
;;-----------------------------------------------------------------------------
;; Copyright (C) ARM Limited, 2007-2014. All rights reserved. 
;;-----------------------------------------------------------------------------


    PRESERVE8
    AREA  BOOT, CODE, READONLY, ALIGN=3

    EXPORT	getCpuNum
    EXPORT	getClusterNum
    EXPORT	getClusterCpuNum
    EXPORT	isCpuDone
    EXPORT	setCpuDone
    EXPORT lock_mutex
    EXPORT unlock_mutex
    EXPORT enqueueData
    EXPORT dequeueData
    EXPORT init_mutex
    EXPORT init_top
    EXPORT init_bottom
    EXPORT getTop
    EXPORT read_lock
    EXPORT getBottom

;; int getCpuNum(void)
;; returns the current Cluster number from PERIPHBASE [31:15] value in CBAR
;getCpuNum
;        ;; which processor is this
;        MRC p15, 4, r0, c15, c0, 0 ; Read Configuration Base Address Register
;        AND r0, r0, #0x1000000
;        LSR r0, r0, #24
;        MOV pc, lr
 
;; int getCpuNum(void)
;; returns the CPU ID bits [1:0] from MPIDR register
getCpuNum
    MRS x0,MPIDR_EL1 ;
    AND x0, x0, #3
    MOV     w0, w0       ; Caller expects a 32-bit value, not a 64-bit
    RET
 
;; int getClusterNum(void)
;; returns Cluster ID [11:8] from MPIDR register shifted to bits [3:0]
getClusterNum
    MRS x0,MPIDR_EL1 ;
    LSR x0, x0, #8
    AND x0, x0, #0xF
    MOV     w0, w0       ; Caller expects a 32-bit value, not a 64-bit
    RET
 
;; int getClusterCpuNum(void)
;; returns CPU ID bits [1:0] and Cluster ID [11:8] from MPIDR register 
getClusterCpuNum
    MRS x0,MPIDR_EL1 ;
    LDR     x1, =0xF03
    AND x0, x0, x1
    MOV     w0, w0       ; Caller expects a 32-bit value, not a 64-bit
    RET
 
; int isCpuDone(int i)
isCpuDone
    IMPORT  ||Image$$GLOBAL$$ZI$$Base|| ; Linker symbol from scatter file
    LDR x1, =||Image$$GLOBAL$$ZI$$Base||
    LDR x2, [x1,x0,LSL#3]
    MOV x0,x2
    MOV     w0, w0       ; Caller expects a 32-bit value, not a 64-bit
    RET

; void setCpuDone(int i)
setCpuDone
    IMPORT  ||Image$$GLOBAL$$ZI$$Base|| ; Linker symbol from scatter file
    LDR x1, =||Image$$GLOBAL$$ZI$$Base||
    MOV x2, #0x1
    STR x2, [x1,x0,LSL#3]
    MOV     w0, w0       ; Caller expects a 32-bit value, not a 64-bit
    RET


locked EQU 1
unlocked EQU 0
lock_offset EQU 20
top_offset EQU 40
bottom_offset EQU 30
size EQU 20
qstart_location EQU 100

;; init_mutex(int)  to initialize the mutex to unlocked

init_mutex
    IMPORT  ||Image$$GLOBAL$$ZI$$Base|| ; Linker symbol from scatter file
    LDR x1, =||Image$$GLOBAL$$ZI$$Base||
    MOV x2, #0x0
    MOV x3,  #lock_offset ;; offset for shared memory
    ADD x3, x3, x1
    STR x2, [x3]  ;; Load value of unlocked into the default lock location
    DMB ISH
    RET

;; read_lock(void) to obtain the value of lock to check if shared resource is locked/unlocked
read_lock
    IMPORT  ||Image$$GLOBAL$$ZI$$Base|| ; Linker symbol from scatter file
    LDR x1, =||Image$$GLOBAL$$ZI$$Base||
    MOV x3,  #lock_offset ;; offset for shared memory
    ADD x3, x3, x1
    LDR x0, [x3]
    MOV     w0, w0       ; Caller expects a 32-bit value, not a 64-bit
    RET
 
;; lock_mutex(int) function is used to lock the shared resource

lock_mutex 
    ;;PUSH     {w4,w20}
    DMB ISH
    IMPORT  ||Image$$GLOBAL$$ZI$$Base|| ; Linker symbol from scatter file
    ;;1 LDREX r2, [r0]
    LDR x1, =||Image$$GLOBAL$$ZI$$Base||
    LDR x2, =locked
    MOV x3,  #lock_offset ;; offset for shared memory
    ADD x3, x3, x1

PLDW
IFUPDATE 
    LDAXR w4, [x3] 
    CMP w4, #unlocked ; Test if mutex is locked or unlocked
    ;;  WFENE                            ; wait if it's locked
    STXR w4, w0, [x3] ; Not locked, attempt to lock it
    CMP w4, #0x0 ; Check if Store-Exclusive failed
    BNE IFUPDATE ; Failed - retry from 1 
    ; Lock acquired
    DMB ISH; Required before accessing protected resource
    ;;POP      {x4,pc}    
    RET

; unlock_mutex(int) function is used to unlock the shared resource

unlock_mutex 
    IMPORT  ||Image$$GLOBAL$$ZI$$Base|| ; Linker symbol from scatter file
    LDR x1, =||Image$$GLOBAL$$ZI$$Base||
    MOV x3,  #lock_offset ;; offset for shared memory
    ADD x3, x3, x1
    LDR x2, =unlocked
    DMB ISH; Required before releasing protected resource
    STR x2, [x3] ; Unlock mutex
    DMB ISH
    RET

;; int init_top(void) store the pointer to top in a location which is accesible in global space and stores top offset with value of queue top
init_top
    IMPORT  ||Image$$GLOBAL$$ZI$$Base|| ; Linker symbol from scatter file
    LDR x1, =||Image$$GLOBAL$$ZI$$Base||
    LDR x2, =top_offset
    ADD x2, x2, x1
    MOV x3, #qstart_location
    ADD x3, x3, x1  ;; store top from the base global object
    STR x3, [x2]  ;; initialize top of the stack to 100 spaces above global
    MOV x0, x3
    MOV     w0, w0       ; Caller expects a 32-bit value, not a 64-bit
    RET

;; int init_bottom(void) to initialize the bottom of the queue
init_bottom
    ;;PUSH     {x4,lr}
    IMPORT  ||Image$$GLOBAL$$ZI$$Base|| ; Linker symbol from scatter file
    LDR x1, =||Image$$GLOBAL$$ZI$$Base||
    LDR x2, =bottom_offset
    ADD x2, x2, x1
    LDR x3, =top_offset
    ADD x3, x3, x1
    LDR x4, [x3]
    STR x4, [x2]
    MOV x0, x4
    ;;POP      {x4,lr}    
    MOV     w0, w0       ; Caller expects a 32-bit value, not a 64-bit
    RET

;; void enqueueData(int) adds data into the top of the queue and increments top

enqueueData
    IMPORT  ||Image$$GLOBAL$$ZI$$Base|| ; Linker symbol from scatter file
    LDR x1, =||Image$$GLOBAL$$ZI$$Base||
    LDR x2, =top_offset
    ADD x2, x2, x1
    LDR x3, [x2]   ;; load the value of top
    STR x0, [x3]   ;; Load the value of value that top points to and return it
    ADD x3, x3, #4   ;; increment top
    STR x3, [x2]  ;; store top back to the top memory location
    DMB ISH
    MOV     w0, w0       ; Caller expects a 32-bit value, not a 64-bit
    RET

;; void dequeueData(int) returns the data from the bottom of the queue 
dequeueData
    IMPORT  ||Image$$GLOBAL$$ZI$$Base|| ; Linker symbol from scatter file
    LDR x1, =||Image$$GLOBAL$$ZI$$Base||
    LDR x2, =bottom_offset
    ADD x2, x2, x1
    LDR x3, [x2]  ;; load the value of bottom address of the queue
    LDR x0, [x3]  ;; ;pad the value of the element of the queue at the bottom address
    ADD x3, x3, #4  ;; increment bottom
    STR x3, [x2]  ;; store bottom
    DMB ISH
    MOV     w0, w0       ; Caller expects a 32-bit value, not a 64-bit
    RET

;; int getTop() used to calculate the top of the queue, useful for calculating qempty and qfull cases
getTop
    IMPORT  ||Image$$GLOBAL$$ZI$$Base|| ; Linker symbol from scatter file
    LDR x1, =||Image$$GLOBAL$$ZI$$Base||
    LDR x2, =top_offset
    ADD x2, x2, x1
    LDR x3, [x2]
    MOV x0, x3
    DMB ISH
    MOV     w0, w0       ; Caller expects a 32-bit value, not a 64-bit
    RET

;; int getBottom() used to calculate the bottom of the queue, useful for calculating qempty and qfull cases
getBottom
    IMPORT  ||Image$$GLOBAL$$ZI$$Base|| ; Linker symbol from scatter file
    LDR x1, =||Image$$GLOBAL$$ZI$$Base||
    LDR x2, =bottom_offset
    ADD x2, x2, x1
    LDR x3, [x2]
    MOV x0, x3
    MOV     w0, w0       ; Caller expects a 32-bit value, not a 64-bit
    RET


   END
