
/*
** Copyright (C) ARM Limited, 2002. All rights reserved.
*/

/*
** This file contains re-implementations of functions whose
** C library implementations rely on semihosting.  
** This includes I/O and clocking functionality.
**
** Defining USE_SERIAL_PORT targets the I/O
** to the Integrator AP serial port A.  Otherwise, I/O is targeted
** to the debugger console.  
** 
** Defining USE_TARGET_CLOCK targets the clocking mechanism to the
** Integrator core module reference clock.  Otherwise, clocking is 
** timed off the host system clock.  
**
** In an image built to run on a standalone target USE_SERIAL_PORT
** USE_HOST_CLOCK must be defined.  
**
*/
#include <stdio.h>


#define TRUE 1
#define FALSE 0


/*
** Importing __use_no_semihosting ensures that our image doesn't link
** with any C Library code that makes direct use of semihosting. 
**
** Build with STANDALONE to include this symbol.
**
*/

#ifdef STANDALONE
#define USE_SERIAL_PORT
asm("  .global __use_no_semihosting\n");
#endif


/*
** Retargeted I/O
** ==============
** The following C library functions make use of semihosting
** to read or write characters to the debugger console: fputc(),
** fgetc(), and _ttywrch().  They must be retargeted to write to
** the Integrator AP UART.  __backspace() must also be retargeted
** with this layer to enable scanf().  See the Compiler and
** Libraries Guide.
*/

/*
** These must be defined to avoid linking in stdio.o from the
** C Library
*/
#ifdef USE_SERIAL_PORT

struct __FILE { int handle;   /* Add whatever you need here */};
FILE __stdout;
FILE __stdin;
FILE __stderr;


/* Following are defined in serial.c */
extern void rpi_putc(char c);

/*
** __backspace must return the last char read to the stream
** fgetc() needs to keep a record of whether __backspace was
** called directly before it
*/
int last_char_read;
int backspace_called;

int fputc(int ch, FILE *f)
{
    char tempch = ch;
 
    (void) rpi_putc(tempch);

    return ch;
}

int fgetc(FILE *f)
{
    // No support to read from terminal :(
    unsigned char tempch = '0';

    return tempch;
}

void _ttywrch(int ch)
{
    char tempch = ch;

    (void) rpi_putc(tempch);

}

/*
** The effect of __backspace() should be to return the last character
** read from the stream, such that a subsequent fgetc() will
** return the same character again.
*/

int __backspace(FILE *f)
{
    backspace_called = TRUE;
    return 0;
}

/* END of Retargeted I/O */
#endif        //end of '#ifdef USE_SERIAL_PORT'



/*
** Exception Signaling and Handling
** ================================
** The C library implementations of ferror() uses semihosting directly
** and must therefore be retargeted.  This is a minimal reimplementation.  
** _sys_exit() is called after the user's main() function has exited.  The C library
** implementation uses semihosting to report to the debugger that the application has
** finished executing.  
*/


#ifdef STANDALONE

int ferror(FILE *f)
{
    return EOF;
}

void _sys_exit(int return_code)
{
    while(1);
}

/* Returns the command line used to invoke the program.
 * cmd       pointer to a buffer to store the command line
 * len       length of the buffer
 * This example assumes main() takes no arguments, so just returns NULL
 */
char *_sys_command_string(char *cmd, int len)
{
    return NULL;
}

#endif        //end of '#ifdef STANDALONE
 
// End of File retarget.c
